#include "conversionthread.h"
#include "mainwindowimpl.h"

float conversionThread::percentage()
{
	return p;
}

void conversionThread::setPercentage( float value )
{
	p = value;
	emit percentageChanged((int)p);
}

void conversionThread::incrementPercentage( float amount )
{
	setPercentage( percentage() + amount);
}

void conversionThread::notify(QString n, bool newline=true)
{
	if (newline)
		emit notification(n + "\n"); else
		emit notification(n);
}

void conversionThread::done( void )
{
	emit finished();
	exit(0);
}

void conversionThread::execCmd ( QString command, QStringList params )
{
	proc.start(command, params);
	if (!proc.waitForFinished())
		notify(tr("cannot execute command ") + command);
}

QString conversionThread::getLabel ( QString fileName, QString label )
{
	proc.start("identify", QStringList()<<"-format"<<label<<fileName);
	QString result;
	if (!proc.waitForFinished()) {
		result = "";
		qDebug()<<tr("Error getting label");
		notify(tr("ERROR FORMATTING LABEL"));
	} else {
		result = proc.readAll();
		result.remove(QChar('\n'));
	}
	return result;
}


void conversionThread::run()
{
	qDebug()<<tr("Created another thread to perform operations in.");
	notify(tr("Stage 1 of 4: Copy the files to a temporary directory"));
	int count = param.files.count();
	int counter;
	param.tempDir.append("/timelapse");
	QString cRes = "";
	cRes.setNum(param.xRes);
	cRes.append("x");
	cRes.append(QString::number(param.yRes));
	QFile curFile;
	execCmd("rm", QStringList()<<"-rf"<<param.tempDir);
	execCmd("mkdir", QStringList()<<param.tempDir);
	QDir::setCurrent(param.tempDir);
	QString newname;
	QStringList newFiles;
	QStringList newFilesPng;
	QFileInfo info;
	for (counter = 0; counter <= (count - 1); counter++) {
		curFile.setFileName(param.files.at(counter));
		newname.setNum(counter + 1);
		while (newname.length() < 9)
			newname.prepend("0");
		info.setFile(param.files.at(counter));
		newFilesPng.append(newname + ".png");
		newname.append(".");
		newname.append(info.suffix().toLower());
		curFile.copy(newname);
		newFiles.append(newname);
		notify(tr("Copied ") + param.files.at(counter) + tr(" to ") + newname);
		curFile.close();
		if ( param.stop ) return;
	}
	notify("Stage 2 of 4: Convert the files");
	for (counter = 0; counter <= (count - 1); counter++) {
		notify(tr("Converting ") + newFiles.at(counter) + tr(" ..."), false);
		if ( param.type == 0 )
			resFlag = ">"; else
			resFlag = "^";
		gravityList.clear();
		gravityList
			<<"center"
			<<"north-west"
			<<"north-east"
			<<"south-west"
			<<"south-east"
			<<"center"
			<<"west"
			<<"east"
			<<"north"
			<<"south";
		gravity = gravityList.at(param.type);
		if ( !param.lossless )
			execCmd("convert", QStringList()<<"-size"<<cRes<<newFiles.at(counter)<<"-thumbnail"<<cRes + resFlag<<"-background"<<param.bgColour<<"-gravity"<<gravity<<"-extent"<<cRes<<newFiles.at(counter)); else {
			execCmd("convert", QStringList()<<"-size"<<cRes<<newFiles.at(counter)<<"-thumbnail"<<cRes + resFlag<<"-background"<<param.bgColour<<"-gravity"<<gravity<<"-extent"<<cRes<<newFilesPng.at(counter));
			execCmd("rm", QStringList()<<"-f"<<newFiles.at(counter));
		}
		if ( param.stop ) return;
		notify(tr("done"));
		if ( param.labelPosition != 0) {
			notify(tr("Adding label to ") + newFiles.at(counter) + tr(" ..."), false);
			gravityList.clear();
			gravityList
				<<"north-west"
				<<"north-east"
				<<"south-west"
				<<"south-east"
				<<"center"
				<<"west"
				<<"east"
				<<"north"
				<<"south";
			gravity = gravityList.at(param.labelPosition - 1);
			if ( !param.lossless )
				execCmd("convert", QStringList()<<newFiles.at(counter)<<"-stroke"<<param.labelColour<<"-fill"<<param.labelColour<<"-font"<<param.labelFont<<"-pointsize"<<QString::number(param.labelSize)<<"-gravity"<<gravity<<"-draw"<<"text 3,3 '" + getLabel(param.files.at(counter), param.label) + "'"<<newFiles.at(counter)); else
				execCmd("convert", QStringList()<<newFilesPng.at(counter)<<"-stroke"<<param.labelColour<<"-fill"<<param.labelColour<<"-font"<<param.labelFont<<"-pointsize"<<QString::number(param.labelSize)<<"-gravity"<<gravity<<"-draw"<<"text 3,3 '" + getLabel(param.files.at(counter), param.label) + "'"<<newFilesPng.at(counter));
			notify(tr("done"));
		}
		if ( param.stop ) return;
	}
	done();
}