#include "mainwindowimpl.h"

//
MainWindowImpl::MainWindowImpl( QWidget * parent, Qt::WFlags f) 
	: QMainWindow(parent, f)
{
	process = new QProcess(this);
	flag = justStarted;
	qDebug()<<tr("TimeLapse video creator by Angel Angelov");
	settingsFile = getenv("HOME");
	settingsFile.append("/.timelapse");
	qDebug()<<tr("Using settings file:");
	qDebug()<<settingsFile;
	setupUi(this);
	tabWidget->setCurrentIndex(2);
	filesTab->setEnabled(false);
	imTab->setEnabled(false);
	startBtn->setEnabled(false);
	cancelBtn->setEnabled(false);
	
	typeCombo->addItem(tr("Pad ( Recommended )"));
	typeCombo->addItem(tr("Crop, upper-left"));
	typeCombo->addItem(tr("Crop, upper-right"));
	typeCombo->addItem(tr("Crop, lower-left"));
	typeCombo->addItem(tr("Crop, lower-right"));
	typeCombo->addItem(tr("Crop, center"));
	typeCombo->addItem(tr("Crop, center-left"));
	typeCombo->addItem(tr("Crop, center-right"));
	typeCombo->addItem(tr("Crop, center-up"));
	typeCombo->addItem(tr("Crop, center-down"));
	
	codecCombo->addItem("mpeg4");
	codecCombo->addItem("mjpeg");
	codecCombo->addItem("4xm");
	codecCombo->addItem("8bps");
	codecCombo->addItem("aasc");
	codecCombo->addItem("amv");
	codecCombo->addItem("asv1");
	codecCombo->addItem("asv2");
	codecCombo->addItem("avs");
	codecCombo->addItem("bethsoftvid");
	codecCombo->addItem("bfi");
	codecCombo->addItem("dvvideo");
	codecCombo->addItem("h264");
	codecCombo->addItem("libtheora");
	codecCombo->addItem("flv");
	codecCombo->addItem("rv40");
	codecCombo->setCurrentIndex(0);
	
	labelPositionCombo->addItem(tr("No label"));
	labelPositionCombo->addItem(tr("upper-left"));
	labelPositionCombo->addItem(tr("upper-right"));
	labelPositionCombo->addItem(tr("lower-left"));
	labelPositionCombo->addItem(tr("lower-right"));
	labelPositionCombo->addItem(tr("center"));
	labelPositionCombo->addItem(tr("center-left"));
	labelPositionCombo->addItem(tr("center-right"));
	labelPositionCombo->addItem(tr("center-up"));
	labelPositionCombo->addItem(tr("center-down"));
	labelPositionCombo->setCurrentIndex(0);
	labelOptionsGroup->setEnabled(false);
	
	connectUi();
	process->start("identify", QStringList() << "--version");
	ok = true;
	if (!process->waitForStarted()) {
		statusText->appendPlainText(tr("Error: No ImageMagick found\n"));
		ok = false;
	} else {
		if (!process->waitForFinished()) {
			statusText->appendPlainText(tr("Error: No ImageMagick found\n")); 
			ok = false;
		} else {
			output = process->readAll();
			qDebug()<<tr("ImageMagick output: ")<<output;
			statusText->appendPlainText(tr("Found ImageMagick."));
			statusText->appendPlainText(output);
			statusText->appendPlainText("\n\n");
		}
	}
	process->start("identify", QStringList() << "-list" << "font");
	if (!process->waitForStarted()) {
		statusText->appendPlainText(tr("Error: Cannot get list of fonts!\n"));
		ok = false;
	} else {
		if (!process->waitForFinished()) {
			statusText->appendPlainText(tr("Error: Cannot get list of fonts!\n")); 
			ok = false;
		} else {
			output = process->readAll();
			qDebug()<<tr("ImageMagick output: ")<<output;
			QStringList fontList = output.split("\n");
			QStringList realFontList;
			QString buf;
			int j;
			for (j = 0; j < fontList.count(); j++) {
				buf = fontList.at(j);
				if (buf.startsWith("  Font: ")) {
					buf.remove(0, 8);
					realFontList.append(buf);
				}
			}
			fontCombo->addItems(realFontList);
		}
	}
	process->start("ffmpeg", QStringList() << "--version");
	if (!process->waitForStarted()) {
		statusText->appendPlainText(tr("Error: No FFmpeg found\n"));
		ok = false;
	} else {
		if (!process->waitForFinished()) {
			statusText->appendPlainText(tr("Error: No FFmpeg found\n")); 
			ok = false;
		} else {
			output = process->readAll();
			qDebug()<<tr("FFmpeg output: ")<<output;
			statusText->appendPlainText(tr("Found FFmpeg."));
			statusText->appendPlainText(output);
			statusText->appendPlainText("\n\n");
		}
	}
	if (ok) {
		filesTab->setEnabled(true);
		imTab->setEnabled(true);
		startBtn->setEnabled(true);
		statusText->appendPlainText(tr("You may now go to the Files tab and select the files you want to convert."));
	}
	readSettings();
}

void MainWindowImpl::connectUi( void )
{
	connect(addBtn, SIGNAL(clicked()), this, SLOT(addClicked()));
	connect(labelPositionCombo, SIGNAL(currentIndexChanged(int)), this, SLOT(enableLabel(int)));
	connect(upBtn, SIGNAL(clicked()), this, SLOT(upClicked()));
	connect(downBtn, SIGNAL(clicked()), this, SLOT(downClicked()));
	connect(removeBtn, SIGNAL(clicked()), this, SLOT(removeClicked()));
	connect(clearBtn, SIGNAL(clicked()), this, SLOT(clearClicked()));
	connect(sortBtn, SIGNAL(clicked()), this, SLOT(sortClicked()));
	connect(outFileBtn, SIGNAL(clicked()), this, SLOT(outFileClicked()));
	connect(tempDirBtn, SIGNAL(clicked()), this, SLOT(tempDirClicked()));
	connect(startBtn, SIGNAL(clicked()), this, SLOT(runClicked()));
	connect(cancelBtn, SIGNAL(clicked()), this, SLOT(cancelClicked()));
	connect(colourBtn, SIGNAL(clicked()), this, SLOT(colourDialog()));
	connect(colourBtn2, SIGNAL(clicked()), this, SLOT(colourDialog2()));
	connect(&myThread, SIGNAL(notification(QString)), this, SLOT(displayNotification(QString)));
	connect(&myThread, SIGNAL(percentageChanged(int)), this, SLOT(changePercentage(int)));
	connect(&myThread, SIGNAL(finished()), this, SLOT(cancelClicked()));
}

void MainWindowImpl::displayNotification ( QString text )
{
	statusText->appendPlainText( text );
}

void MainWindowImpl::changePercentage ( int value )
{
	overallBar->setValue( value );
}

uint MainWindowImpl::getFileCreated( QString fileName )
{
	QFileInfo info (fileName);
	QDateTime dateTime = info.lastModified();
	uint result = dateTime.toTime_t();
	#ifdef _SUPER_VERBOSE
		qDebug()<<"Date of creation for file "<<fileName<<" is "<<result<<" and HR is "<<dateTime.toString("h:m:s ap dd.MM.yyyy");
	#endif
	return result;
}

void MainWindowImpl::enableLabel ( int pos )
{
	if ( pos == 0)
		labelOptionsGroup->setEnabled(false); else
		labelOptionsGroup->setEnabled(true);
}

void MainWindowImpl::closeEvent(QCloseEvent *event)
{
	saveSettings();
	qDebug()<<tr("Closing.");
	event->accept();
}

QString MainWindowImpl::fReadLine(QFile *file)
{
	QString result = file->readLine();
	result.remove("\n", Qt::CaseInsensitive);
	return result;
}

bool MainWindowImpl::fReadBool(QFile *file)
{
	QString s = file->readLine();
	if (s.startsWith("true", Qt::CaseInsensitive))
		return true;
	//else
	return false;
}

int MainWindowImpl::fReadInt(QFile *file)
{
	QString s = file->readLine();
	bool ok;
	int result = s.toInt(&ok, 10);
	if (ok)
		return result;
	else {
		qDebug()<<tr("Can't deciphre int value. Brrrrr!");
		return 0;
	}
}

void MainWindowImpl::colourDialog ( void )
{
	QColor colour = QColorDialog::getColor();
	labelColourEdit->setText(colour.name());
}

void MainWindowImpl::colourDialog2 ( void )
{
	QColor colour = QColorDialog::getColor();
	bgColourEdit->setText(colour.name());
}

void MainWindowImpl::saveSettings ( void )
{
	QFile f (settingsFile);
	if (!f.open(QIODevice::WriteOnly | QIODevice::Text)) {
		qDebug()<<tr("Can't write settings file. Perhaps permissions in home directory are insufficient. OH NO! Big Problem!!!");
		QMessageBox::critical(this, tr("Aaaaaargh"), tr("Can't write config file. Bye :-("));
		return;
	}
	qDebug()<<tr("Writing settings to ~/.timelapse");
	f.write(QByteArray::number(xSpin->value(), 10));
	f.write("\n");
	f.write(QByteArray::number(ySpin->value(), 10));
	f.write("\n");
	f.write(QByteArray::number(typeCombo->currentIndex(), 10));
	f.write("\n");
	f.write(QByteArray::number(labelPositionCombo->currentIndex(), 10));
	f.write("\n");
	f.write(QByteArray::number(fontCombo->currentIndex(), 10));
	f.write("\n");
	f.write(QByteArray::number(labelSizeSpin->value(), 10));
	f.write("\n");
	f.write(labelColourEdit->text().toAscii());
	f.write("\n");
	f.write(labelTextEdit->text().toAscii());
	f.write("\n");
	f.write(QByteArray::number(chooseLabelCombo->currentIndex(), 10));
	f.write("\n");
	f.write(QByteArray::number(frameRateSpin->value(), 10));
	f.write("\n");
	f.write(QByteArray::number(bitRateSpin->value(), 10));
	f.write("\n");
	f.write(QByteArray::number(codecCombo->currentIndex(), 10));
	f.write("\n");
	f.write(otherEdit->text().toAscii());
	f.write("\n");
	if (losslessRadio->isChecked())
		f.write("true\n"); else
		f.write("false\n");
	f.write(tempDirEdit->text().toAscii());
	f.write("\n");
	f.write(outFileEdit->text().toAscii());
	f.write("\n");
	f.write(bgColourEdit->text().toAscii());
	f.write("\n");
	f.close();
	qDebug()<<tr("Done.");
}

void MainWindowImpl::readSettings ( void )
{
	QFile f (settingsFile);
	if (!f.open(QIODevice::ReadOnly | QIODevice::Text)) {
		qDebug()<<tr("Can't read settings file. Possible first run. Proceeding quietly");
		return;
	}
	qDebug()<<tr("Reading settings from ~/.timelapse");
	xSpin->setValue(fReadInt(&f));
	ySpin->setValue(fReadInt(&f));
	typeCombo->setCurrentIndex(fReadInt(&f));
	labelPositionCombo->setCurrentIndex(fReadInt(&f));
	fontCombo->setCurrentIndex(fReadInt(&f));
	labelSizeSpin->setValue(fReadInt(&f));
	labelColourEdit->setText(fReadLine(&f));
	labelTextEdit->setText(fReadLine(&f));
	chooseLabelCombo->setCurrentIndex(fReadInt(&f));
	frameRateSpin->setValue(fReadInt(&f));
	bitRateSpin->setValue(fReadInt(&f));
	codecCombo->setCurrentIndex(fReadInt(&f));
	otherEdit->setText(fReadLine(&f));
	losslessRadio->setChecked(fReadBool(&f));
	tempDirEdit->setText(fReadLine(&f));
	outFileEdit->setText(fReadLine(&f));
	bgColourEdit->setText(fReadLine(&f));
	f.close();
	qDebug()<<tr("Done.");
}

void MainWindowImpl::refreshList ( void )
{
	filesList->clear();
	filesList->addItems(inFiles);
}

void MainWindowImpl::addClicked ( void )
{
	inFiles.append(QFileDialog::getOpenFileNames(this, tr("Select one or more images to add"), "", tr("Images (*.jpg *.xpm *.png)")));
	refreshList();
}

void MainWindowImpl::removeClicked ( void )
{
	int cur = filesList->currentRow();
	inFiles.removeAt(cur);
	refreshList();
	if (cur != inFiles.count())
		filesList->setCurrentRow(cur);
	else
		filesList->setCurrentRow(inFiles.count() - 1);
}

void MainWindowImpl::clearClicked ( void )
{
	inFiles.clear();
	refreshList();
}

void MainWindowImpl::upClicked ( void )
{
	int cur = filesList->currentRow();
	if (cur <= 0)
		return;
	inFiles.swap(cur, cur - 1);
	refreshList();
	filesList->setCurrentRow(cur - 1);
}

void MainWindowImpl::downClicked ( void )
{
	int cur = filesList->currentRow();
	if (cur >= inFiles.count() - 1)
		return;
	inFiles.swap(cur, cur + 1);
	refreshList();
	filesList->setCurrentRow(cur + 1);
}

void MainWindowImpl::sortClicked ( void )
{
	uint test1;
	uint test2;
	bool hasBeenSwapping;
	int i;
	do {
		hasBeenSwapping = false;
		#ifdef _SUPER_VERBOSE
			qDebug()<<"Start sort!";
		#endif
		for (i = 0; i < inFiles.count() - 1; i++) {
			test1 = getFileCreated(inFiles.value(i));
			test2 = getFileCreated(inFiles.value(i + 1));
			#ifdef _SUPER_VERBOSE
				qDebug()<<"Test1: "<<test1<<" Test1 file: "<<inFiles.value(i)<<" Test2: "<<test2<<" Test2 file: "<<inFiles.value(i + 1);
			#endif
			if (test1 > test2) {
				inFiles.swap(i, i + 1);
				hasBeenSwapping = true;
				#ifdef _SUPER_VERBOSE
					qDebug()<<"SWAP!: "<<test1<<"(Test1) > "<<test2<<"(Test2)";
				#endif
			}
		}
	} while (hasBeenSwapping);
	refreshList();
}

void MainWindowImpl::outFileClicked ( void )
{
	outFileEdit->setText(QFileDialog::getSaveFileName(this, tr("Output file:"), outFileEdit->text(), tr("Mpeg4 (*.mp4 *.mpeg4 *.mpg *.mpeg);;Windows AVI (*.avi);;All files (*.*)")));
}

void MainWindowImpl::tempDirClicked ( void )
{
	tempDirEdit->setText(QFileDialog::getExistingDirectory(this, tr("Select TEMPDIR"), tempDirEdit->text(), QFileDialog::ShowDirsOnly));
}

void MainWindowImpl::runClicked ( void )
{
	myThread.param.xRes = xSpin->value();
	myThread.param.yRes = ySpin->value();
	myThread.param.type = typeCombo->currentIndex();
	myThread.param.bgColour = bgColourEdit->text();
	myThread.param.frameRate = frameRateSpin->value();
	myThread.param.bitRate = bitRateSpin->value();
	myThread.param.codec = codecCombo->itemText(codecCombo->currentIndex());
	myThread.param.otherParams = otherEdit->text();
	myThread.param.labelPosition = labelPositionCombo->currentIndex();
	myThread.param.labelSize = labelSizeSpin->value();
	myThread.param.labelColour = labelColourEdit->text();
	myThread.param.label = labelTextEdit->text();
	myThread.param.labelFont = fontCombo->itemText(fontCombo->currentIndex());
	myThread.param.lossless = losslessRadio->isChecked();
	myThread.param.tempDir = tempDirEdit->text();
	myThread.param.files = inFiles;
	myThread.param.outFile = outFileEdit->text();
	myThread.param.stop = false;
	
	statusText->clear();
	statusText->appendPlainText(tr("Starting procedure\n"));
	qDebug()<<tr("Starting thread...");
	startBtn->setEnabled(false);
	tabWidget->setCurrentIndex(2);
	filesTab->setEnabled(false);
	imTab->setEnabled(false);
	cancelBtn->setEnabled(true);
	myThread.start();
}

void MainWindowImpl::cancelClicked ( void )
{
	myThread.param.stop = true;
	process->start("killall", QStringList()<<"convert");
	process->waitForFinished();
	process->start("killall", QStringList()<<"ffmpeg");
	process->waitForFinished();
	int w = 0;
	while ( (w != 10) && (!myThread.isFinished())) {
		sleep ( 1 );
	};
	if (!myThread.isFinished())
		exit ( 1 );
	startBtn->setEnabled(true);
	filesTab->setEnabled(true);
	imTab->setEnabled(true);
	cancelBtn->setEnabled(false);
	process->start("rm", QStringList()<<"-rf"<<tempDirEdit->text() + "/timelapse");
	process->waitForFinished();
	statusText->appendPlainText(tr("Finished\n"));
}
//
